@@ -1,76 +1,10 @@
 import os
-from gaffer.commands import get_commands
+from gaffer.node.commands import get_commands
 
 _HEADER = """\
 .. _cli:
 
-==============
-Console tools
-==============
-
-gafferd
-=======
-
-Gafferd is a server able to launch and manage processes. It can be
-controlled via the :doc:<http> .
-
-Usage
-+++++
-
-::
-
-    $ gafferd --help
-    usage: gafferd [-h] [--daemon] [--pidfile PIDFILE] config
-
-    Run some watchers.
-
-    positional arguments:
-      config             configuration file
-
-    optional arguments:
-      -h, --help         show this help message and exit
-      --daemon           Start gaffer in the background
-      --pidfile PIDFILE
-
-Config file example
-+++++++++++++++++++
-
-::
-
-    [gaffer]
-    http_endpoints = public
-
-    [endpoint:public]
-    bind = 127.0.0.1:5000
-    ;certfile=
-    ;keyfile=
-
-    [webhooks]
-    ;create = http://some/url
-    ;proc.dummy.spawn = http://some/otherurl
-
-
-    [process:dummy]
-    cmd = ./dummy.py
-    ;cwd = .
-    ;uid =
-    ;gid =
-    ;detach = false
-    ;shell = false
-    ; flapping format: attempts=2, window=1., retry_in=7., max_retry=5
-    ;flapping = 2, 1., 7., 5
-    numprocesses = 1
-    redirect_output = stdout, stderr
-    ; redirect_input  = true
-    ; graceful_timeout = 30
-
-    [process:echo]
-    cmd = ./echo.py
-    numprocesses = 1
-    redirect_output = stdout, stderr
-    redirect_input  = true
-
-gafferctl
+Gafferctl
 =========
 
 *gafferctl* can be used to run any command listed below. For
@@ -87,6 +21,8 @@ You can create your own client either by using the client API provided
 in the httpclient module or by reading the doc here and passing your own
 message to the gaffer node. All messages are encoded in JSON.
 
+.. image:: _static/gaffer_watch.png
+
 Usage
 +++++
 
@@ -126,7 +62,7 @@ def generate_commands(app):
     if not os.path.exists(path):
         os.makedirs(path)
 
-    tocname = os.path.join(app.srcdir, "commands%s" % ext)
+    tocname = os.path.join(app.srcdir, "gafferctl%s" % ext)
 
     with open(tocname, "w") as toc:
         toc.write(_HEADER)
