@@ -11,11 +11,18 @@ Classes
 
 """
 from collections import deque
+import copy
+import operator
 from threading import RLock
 
 import pyuv
 import six
 
+try:
+    from collections import OrderedDict
+except ImportError:
+    from .datastructures import OrderedDict
+
 from .events import EventEmitter
 from .process import Process
 from .state import ProcessState, ProcessTracker
@@ -84,8 +91,9 @@ class Manager(object):
         self.started = False
         self._stop_ev = None
         self.max_process_id = 0
-        self.processes = {}
-        self.running = {}
+        self.processes = OrderedDict()
+        self.running = OrderedDict()
+        self.groups = {}
         self.channel = deque()
         self._updates = deque()
         self._signals = []
@@ -126,7 +134,6 @@ class Manager(object):
         self._signals.append("STOP")
         self.wakeup()
 
-
     def restart(self, callback=None):
         """ restart all processes in the manager. This function is
         threadsafe """
@@ -134,6 +141,15 @@ class Manager(object):
         self._signals.append("RESTART")
         self.wakeup()
 
+    def start_processes(self):
+        """ start all processes """
+        self._lock.acquire()
+        for name in self.processes:
+            self._lock.release()
+            self.start_process(name)
+            self._lock.acquire()
+        self._lock.release()
+
     def stop_processes(self):
         """ stop all processes in the manager """
         with self._lock:
@@ -167,11 +183,46 @@ class Manager(object):
         self._emitter.subscribe_once(evtype, listener)
     once = subscribe
 
-
     def unsubscribe(self, evtype, listener):
         """ unsubscribe from the event *eventype* """
         self._emitter.unsubscribe(evtype, listener)
 
+    def get_groups(self):
+        """ return the groups list """
+        with self._lock:
+            return list(self.groups)
+
+    def get_group(self, groupname):
+        """ return list of named process of this group """
+        with self._lock:
+            if groupname not in self.groups:
+                raise KeyError('%r not found')
+            return copy.copy(self.groups[groupname])
+
+    def remove_group(self, groupname):
+        """ remove a group and all its processes. All processes are
+        stopped """
+        self._apply_group_func(groupname, self.remove_process)
+
+        # finally remove the group
+        with self._lock:
+            del self.groups[groupname]
+
+    def start_group(self, groupname):
+        """ start all process templates of the group """
+        self._apply_group_func(groupname, self.start_process)
+
+    def stop_group(self, groupname):
+        """ stop all processes templates of the group """
+        self._apply_group_func(groupname, self.stop_process)
+
+    def restart_group(self, groupname):
+        """ restart all processes in a group  """
+        self._apply_group_func(groupname, self.restart_process)
+
+
+
+
     # ------------- process functions
 
     def add_process(self, name, cmd, **kwargs):
@@ -225,12 +276,30 @@ class Manager(object):
             else:
                 start = True
 
+            # Grouped process are prefixed by the name of the group
+            # <grouname>:<name>
+            if ":" in name:
+                group = name.split(":", 1)[0]
+                kwargs['group'] = group
+            else:
+                group = None
+
             state = ProcessState(name, cmd, **kwargs)
             self.processes[name] = state
 
+            # register this name to the group
+            if group is not None:
+                try:
+                    self.groups[group].append(name)
+                except KeyError:
+                    self.groups[group] = [name]
+
             self._publish("create", name=name)
             if start:
-                self.start_process(name)
+                self._publish("start", name=name)
+                self._publish("proc.%s.start" % name, name=name)
+                self._manage_processes(state)
+
 
     def update_process(self, name, cmd, **kwargs):
         """ update a process information.
@@ -298,8 +367,16 @@ class Manager(object):
 
             # stop all processes
             self._stop_processes(name)
-            # remove it the list
-            del self.processes[name]
+
+            # remove it the from the list
+            state = self.processes.pop(name)
+            # also remove it from the group if any.
+            if state.group is not None:
+                if state.group in self.groups:
+                    g = self.groups[state.group]
+                    del g[operator.indexOf(g, name)]
+                    self.groups[state.group] = g
+
             # notify other that this template has been deleted
             self._publish("delete", name=name)
 
@@ -311,6 +388,14 @@ class Manager(object):
             self._manage_processes(state)
 
 
+    def get_process(self, name_or_pid):
+        with self._lock:
+            if isinstance(name_or_pid, int):
+                return self.running[name_or_pid]
+            else:
+                return self.processes[name_or_pid]
+
+
     def get_process_info(self, name):
         """ get process info """
         with self._lock:
@@ -499,7 +584,6 @@ class Manager(object):
     def _stop_processes(self, name):
         """ stop all processes in a template """
         if name not in self.processes:
-            print("return")
             return
 
         # get the template
@@ -525,8 +609,9 @@ class Manager(object):
                 break
 
             # notify  other that the process is beeing stopped
+            self._publish("stop_pid", name=p.name, pid=p.id, os_pid=p.pid)
             self._publish("proc.%s.stop_pid" % p.name, name=p.name,
-                    pid=p.id)
+                    pid=p.id, os_pid=p.pid)
 
             # remove the pid from the running processes
             if p.id in self.running:
@@ -558,7 +643,9 @@ class Manager(object):
         self._tracker.check(p, state.graceful_timeout)
 
         # notify  other that the process is beeing stopped
-        self._publish("proc.%s.stop_pid" % p.name, name=p.name, pid=pid)
+        self._publish("stop_pid", name=p.name, pid=pid, os_pid=p.pid)
+        self._publish("proc.%s.stop_pid" % p.name, name=p.name, pid=pid,
+                os_pid=p.pid)
 
     def _spawn_process(self, state):
         """ spawn a new process and add it to the state """
@@ -575,8 +662,10 @@ class Manager(object):
         # we keep a list of all running process by id here
         self.running[pid] = p
 
+        self._publish("spawn", name=p.name, pid=pid,
+                detached=p.detach, os_pid=p.pid)
         self._publish("proc.%s.spawn" % p.name, name=p.name, pid=pid,
-                detached=p.detach)
+                detached=p.detach, os_pid=p.pid)
 
     def _spawn_processes(self, state):
         """ spawn all processes for a state """
@@ -606,8 +695,9 @@ class Manager(object):
                 self._tracker.check(p, state.graceful_timeout)
 
                 # notify others that the process is beeing reaped
+                self._publish("reap", name=p.name, pid=p.id, os_pid=p.pid)
                 self._publish("proc.%s.reap" % p.name, name=p.name,
-                    pid=p.id)
+                    pid=p.id, os_pid=p.pid)
 
     def _manage_processes(self, state):
         if len(state.running) < state.numprocesses:
@@ -630,23 +720,19 @@ class Manager(object):
         if not check_flapping:
             self._publish("flap", name=state.name)
             # stop the processes
-            self._stop_byname_unlocked(state.name)
+            self._stop_processes(state.name)
             if can_retry:
                 # if we can retry later then set a callback
                 def flapping_cb(handle):
-                    handle.stop()
-
                     # allows respawning
                     state.stopped = False
                     state._flapping_timer = None
 
                     # restart processes
                     self._restart_processes(state)
-
                 # set a callback
                 t = pyuv.Timer(self.loop)
-                t.start(flapping_cb, state.flapping.retry_in,
-                        state.flapping.retry_in)
+                t.start(flapping_cb, state.flapping.retry_in, 0.0)
                 state._flapping_timer = t
             return False
         return True
@@ -657,6 +743,20 @@ class Manager(object):
         self._emitter.publish(evtype, event)
 
 
+    def _apply_group_func(self, groupname, func):
+        self._lock.acquire()
+        if groupname not in self.groups:
+            raise KeyError('%r not found')
+
+        for name in self.groups[groupname]:
+            if name in self.processes:
+                self._lock.release()
+                func(name)
+                self._lock.acquire()
+
+        self._lock.release()
+
+
     # ------------- events handler
 
     def _on_wakeup(self, handle):
@@ -671,12 +771,13 @@ class Manager(object):
             self._restart()
 
     def _on_exit(self, process, exit_status, term_signal):
-        # exit callback called when a process exit
-
         # notify other that the process exited
-        self._publish("proc.%s.exit" % process.name, name=process.name,
-                pid=process.id, exit_status=exit_status,
-                term_signal=term_signal)
+        ev_details = dict(name=process.name, pid=process.id,
+                exit_status=exit_status, term_signal=term_signal,
+                os_pid=process.pid)
+
+        self._publish("exit", **ev_details)
+        self._publish("proc.%s.exit" % process.name, **ev_details)
 
         with self._lock:
             # maybe uncjeck this process from the tracker
@@ -689,12 +790,13 @@ class Manager(object):
 
             if process.id in self.running:
                 self.running.pop(process.id)
-            state.remove(process)
 
             # this template has been removed, return
             if not state:
                 return
 
+            state.remove(process)
+
             # eventually restart the process
             if not state.stopped:
                 # manage the template, eventually restart a new one.
